@ECHO OFF

:: released 2010 March 25

:: SYNOPSIS
::
:: for_each_computer <command> <match...> [ /EXCLUDE <exclusion...> ]
:: 
:: OVERVIEW
::
:: Runs a command on each computer matching a name pattern.  You can also exclude items.
::
:: USAGE
:: 
::  <command> is a command to run.  First argument will be name of computer with double-backslash prefix.
::  <match...> is one or more patterns, separated by spaces.  Each pattern will be compared against the list of active computers, and matches are added to the list of targets.
::  If the optional /EXCLUDE switch is used, <exclusion...> is one or more patterns, separated by spaces.  Each pattern will be compared against the list of targets, and filter out any matches.
::  Pattern matching is a simple substring compare.  Wildcards, regular expressions, etc., are not supported.
::
:: WARNINGS
::
::  Command must be a single word.  Anything complicated has to be wrapped in a batch file.
::  Computers with spaces in their names are likely to cause problems.
::  Dependant on the NetBIOS browse list (Network Neighborhood, My Network Places, etc.) to get active computers.
::
:: TIPS
::
::  De-duplication requires a routine to obtain only unique lines.  If UNIQ.EXE is in PATH, it will be used.  If not, we fall back to a routine implemented using only native Windows commands.  UNIQ.EXE will be much faster if if there are lots of matching computers.  You can get UNIQ.EXE from http://unxutils.sf.net/ or other Unix-compatability toolkits.
::  Use "ECHO" as the command to see what the list of matching computers is before running a command.
::
:: NOTICES
:: 
:: Written by Ben Scott
:: Provided free-of-charge, "AS IS", WITHOUT WARRANTY.  USE AT YOUR OWN RISK.

SETLOCAL ENABLEEXTENSIONS

:: ########################################
:: constants

:: temporary files (base names and/or paths)
SET FEC_LIST_NAME=for_each_computer_list.TMP
SET FEC_OTHER_NAME=for_each_computer_other.TMP
SET FEC_LIST=%TEMP%\%FEC_LIST_NAME%
SET FEC_OTHER=%TEMP%\%FEC_OTHER_NAME%

:: ########################################
:: main program

:: ----------------------------------------
:: initialize

:: clean up any leftover temp files
CALL :del_temp_files

:: ----------------------------------------
:: arguments

:: get command to run
IF "%1"=="" (
	ECHO ERROR: Missing argument.  Specify command to run.
	EXIT /B
)
SET CMD_TO_RUN=%1
SHIFT

:: make sure user specified at least one name pattern to match against
IF "%1"=="" (
	ECHO ERROR: Missing argument.  Specify one or more computer name match patterns.
	EXIT /B
)

:: ----------------------------------------
:: generate list of matching network computers

ECHO Getting list of active network computers...
NET VIEW > "%FEC_OTHER%"

ECHO Finding matching names from list...
:find_next_target
FINDSTR /I /C:%1 "%FEC_OTHER%" >> "%FEC_LIST%"
SHIFT
IF /I "%1"=="/EXCLUDE" GOTO exclude_targets
IF /I "%1"=="/Exclude" GOTO exclude_targets
IF /I "%1"=="/exclude" GOTO exclude_targets
IF NOT "%1"=="" GOTO find_next_target
GOTO after_exclude

:: ----------------------------------------
:: process exclude  list

:exclude_targets
ECHO Excluding matching names from list...
:: remove /EXCLUDE from arg list
SHIFT
:: make sure user gave us something to exclude
IF "%1"=="" (
	ECHO ERROR: Option /EXCLUDE given without patterns to exclude.
	EXIT /B
)
:exclude_next
:: for each exclude pattern, previously output file becomes input file
CALL :shift_files
FINDSTR /V /I /C:%1 "%FEC_OTHER%" > "%FEC_LIST%"
SHIFT
IF NOT "%1"=="" GOTO exclude_next
GOTO after_exclude

:after_exclude

:: ----------------------------------------
:: make sure lines are unique

ECHO De-duplicating...
CALL :shift_files

:: see if we have UNIQ.EXE for faster de-dupe
CALL :have_program UNIQ.EXE
IF ERRORLEVEL 1 GOTO internal_uniq

:: do de-dupe the fast and easy way
SORT "%FEC_OTHER%" | uniq > "%FEC_LIST%"
GOTO after_dedupe

:internal_uniq
:: okay, do it the ugly hard slow way, using native Windows tools only
ECHO NOTE: Using internal uniq routine.  Would be faster with UNIQ.EXE in PATH.
:: create empty output file, FIND command line will abort if file does not exist
TYPE NUL > "%FEC_LIST%"
:: for each line in the input file...
FOR /F "usebackq tokens=* delims= " %%A in ("%FEC_OTHER%") do (
	:: does line exist in output file?
	FIND "%%A" < "%FEC_LIST%" > NUL
	:: if not, copy to output file
	IF ERRORLEVEL 1 ECHO %%A >> "%FEC_LIST%"
)
GOTO after_dedupe

:after_dedupe

:: ----------------------------------------
:: take action

ECHO Performing action on remaining computers (if any)...
SET COUNT=0
FOR /F "usebackq" %%T IN ("%FEC_LIST%") DO (
	CALL %CMD_TO_RUN% %%T
	SET /A COUNT=COUNT + 1
)

:: ----------------------------------------
:: finish

ECHO Processed %COUNT% computers.

CALL :del_temp_files

:: end of main program
EXIT /B

:: ########################################
:: support sub-routines

:: ----------------------------------------
:del_temp_files
:: delete  temp files
CALL :del_if_exist "%FEC_LIST%"
CALL :del_if_exist "%FEC_OTHER%"
:: return to CALLer
EXIT /B

:: ----------------------------------------
:shift_files
:: moves FEC_OTHER to FEC_LIST, deleting the old FEC_LIST
CALL :del_if_exist "%FEC_OTHER%"
RENAME "%FEC_LIST%" "%FEC_OTHER_NAME%"
:: return to CALLer
EXIT /B

:: ----------------------------------------
:del_if_exist
:: deletes a file, if it exists
IF EXIST "%1" DEL "%1"
:: return to CALLer
EXIT /B

:: ----------------------------------------
:have_program
:: determine if we have a program file in PATH
:: returns success if true, failure if false
SET require_program_test_path=
FOR %%i IN (%1) DO SET require_program_test_path=%%~$PATH:i
IF NOT DEFINED require_program_test_path (
	EXIT /B 1
	)
EXIT /B 0

:: ########################################
:: END OF FILE
:: ########################################
